@@ -2,9 +2,6 @@ package encoding
 
 import (
 	"bytes"
-	"crypto/aes"
-	"crypto/cipher"
-	"crypto/md5"
 	"crypto/sha256"
 	"encoding/binary"
 	"hash/fnv"
@@ -17,6 +14,7 @@ import (
 	"github.com/xtls/xray-core/common/buf"
 	"github.com/xtls/xray-core/common/crypto"
 	"github.com/xtls/xray-core/common/drain"
+	"github.com/xtls/xray-core/common/errors"
 	"github.com/xtls/xray-core/common/net"
 	"github.com/xtls/xray-core/common/protocol"
 	"github.com/xtls/xray-core/common/task"
@@ -76,7 +74,7 @@ func (h *SessionHistory) removeExpiredEntries() error {
 	defer h.Unlock()
 
 	if len(h.cache) == 0 {
-		return newError("nothing to do")
+		return errors.New("nothing to do")
 	}
 
 	for session, expire := range h.cache {
@@ -102,10 +100,6 @@ type ServerSession struct {
 	responseBodyIV  [16]byte
 	responseWriter  io.Writer
 	responseHeader  byte
-
-	isAEADRequest bool
-
-	isAEADForced bool
 }
 
 // NewServerSession creates a new ServerSession, using the given UserValidator.
@@ -117,17 +111,12 @@ func NewServerSession(validator *vmess.TimedUserValidator, sessionHistory *Sessi
 	}
 }
 
-// SetAEADForced sets isAEADForced for a ServerSession.
-func (s *ServerSession) SetAEADForced(isAEADForced bool) {
-	s.isAEADForced = isAEADForced
-}
-
 func parseSecurityType(b byte) protocol.SecurityType {
 	if _, f := protocol.SecurityType_name[int32(b)]; f {
 		st := protocol.SecurityType(b)
 		// For backward compatibility.
 		if st == protocol.SecurityType_UNKNOWN {
-			st = protocol.SecurityType_LEGACY
+			st = protocol.SecurityType_AUTO
 		}
 		return st
 	}
@@ -140,7 +129,7 @@ func (s *ServerSession) DecodeRequestHeader(reader io.Reader, isDrain bool) (*pr
 
 	drainer, err := drain.NewBehaviorSeedLimitedDrainer(int64(s.userValidator.GetBehaviorSeed()), 16+38, 3266, 64)
 	if err != nil {
-		return nil, newError("failed to initialize drainer").Base(err)
+		return nil, errors.New("failed to initialize drainer").Base(err)
 	}
 
 	drainConnection := func(e error) error {
@@ -157,7 +146,7 @@ func (s *ServerSession) DecodeRequestHeader(reader io.Reader, isDrain bool) (*pr
 	}()
 
 	if _, err := buffer.ReadFullFrom(reader, protocol.IDBytesLen); err != nil {
-		return nil, newError("failed to read request header").Base(err)
+		return nil, errors.New("failed to read request header").Base(err)
 	}
 
 	var decryptor io.Reader
@@ -177,40 +166,20 @@ func (s *ServerSession) DecodeRequestHeader(reader io.Reader, isDrain bool) (*pr
 		if errorReason != nil {
 			if shouldDrain {
 				drainer.AcknowledgeReceive(bytesRead)
-				return nil, drainConnection(newError("AEAD read failed").Base(errorReason))
+				return nil, drainConnection(errors.New("AEAD read failed").Base(errorReason))
 			} else {
-				return nil, drainConnection(newError("AEAD read failed, drain skipped").Base(errorReason))
+				return nil, drainConnection(errors.New("AEAD read failed, drain skipped").Base(errorReason))
 			}
 		}
 		decryptor = bytes.NewReader(aeadData)
-		s.isAEADRequest = true
-
-	case errorAEAD == vmessaead.ErrNotFound:
-		userLegacy, timestamp, valid, userValidationError := s.userValidator.Get(buffer.Bytes())
-		if !valid || userValidationError != nil {
-			return nil, drainConnection(newError("invalid user").Base(userValidationError))
-		}
-		if s.isAEADForced {
-			return nil, drainConnection(newError("invalid user: VMessAEAD is enforced and a non VMessAEAD connection is received. You can still disable this security feature with environment variable xray.vmess.aead.forced = false . You will not be able to enable legacy header workaround in the future."))
-		}
-		if s.userValidator.ShouldShowLegacyWarn() {
-			newError("Critical Warning: potentially invalid user: a non VMessAEAD connection is received. From 2022 Jan 1st, this kind of connection will be rejected by default. You should update or replace your client software now. This message will not be shown for further violation on this inbound.").AtWarning().WriteToLog()
-		}
-		user = userLegacy
-		iv := hashTimestamp(md5.New(), timestamp)
-		vmessAccount = userLegacy.Account.(*vmess.MemoryAccount)
-
-		aesStream := crypto.NewAesDecryptionStream(vmessAccount.ID.CmdKey(), iv)
-		decryptor = crypto.NewCryptionReader(aesStream, reader)
-
 	default:
-		return nil, drainConnection(newError("invalid user").Base(errorAEAD))
+		return nil, drainConnection(errors.New("invalid user").Base(errorAEAD))
 	}
 
 	drainer.AcknowledgeReceive(int(buffer.Len()))
 	buffer.Clear()
 	if _, err := buffer.ReadFullFrom(decryptor, 38); err != nil {
-		return nil, newError("failed to read request header").Base(err)
+		return nil, errors.New("failed to read request header").Base(err)
 	}
 
 	request := &protocol.RequestHeader{
@@ -225,15 +194,7 @@ func (s *ServerSession) DecodeRequestHeader(reader io.Reader, isDrain bool) (*pr
 	sid.key = s.requestBodyKey
 	sid.nonce = s.requestBodyIV
 	if !s.sessionHistory.addIfNotExits(sid) {
-		if !s.isAEADRequest {
-			drainErr := s.userValidator.BurnTaintFuse(fixedSizeAuthID[:])
-			if drainErr != nil {
-				return nil, drainConnection(newError("duplicated session id, possibly under replay attack, and failed to taint userHash").Base(drainErr))
-			}
-			return nil, drainConnection(newError("duplicated session id, possibly under replay attack, userHash tainted"))
-		} else {
-			return nil, newError("duplicated session id, possibly under replay attack, but this is a AEAD request")
-		}
+		return nil, errors.New("duplicated session id, possibly under replay attack, but this is a AEAD request")
 	}
 
 	s.responseHeader = buffer.Byte(33)             // 1 byte
@@ -257,26 +218,12 @@ func (s *ServerSession) DecodeRequestHeader(reader io.Reader, isDrain bool) (*pr
 
 	if paddingLen > 0 {
 		if _, err := buffer.ReadFullFrom(decryptor, int32(paddingLen)); err != nil {
-			if !s.isAEADRequest {
-				burnErr := s.userValidator.BurnTaintFuse(fixedSizeAuthID[:])
-				if burnErr != nil {
-					return nil, newError("failed to read padding, failed to taint userHash").Base(burnErr).Base(err)
-				}
-				return nil, newError("failed to read padding, userHash tainted").Base(err)
-			}
-			return nil, newError("failed to read padding").Base(err)
+			return nil, errors.New("failed to read padding").Base(err)
 		}
 	}
 
 	if _, err := buffer.ReadFullFrom(decryptor, 4); err != nil {
-		if !s.isAEADRequest {
-			burnErr := s.userValidator.BurnTaintFuse(fixedSizeAuthID[:])
-			if burnErr != nil {
-				return nil, newError("failed to read checksum, failed to taint userHash").Base(burnErr).Base(err)
-			}
-			return nil, newError("failed to read checksum, userHash tainted").Base(err)
-		}
-		return nil, newError("failed to read checksum").Base(err)
+		return nil, errors.New("failed to read checksum").Base(err)
 	}
 
 	fnv1a := fnv.New32a()
@@ -285,46 +232,40 @@ func (s *ServerSession) DecodeRequestHeader(reader io.Reader, isDrain bool) (*pr
 	expectedHash := binary.BigEndian.Uint32(buffer.BytesFrom(-4))
 
 	if actualHash != expectedHash {
-		if !s.isAEADRequest {
-			Autherr := newError("invalid auth, legacy userHash tainted")
-			burnErr := s.userValidator.BurnTaintFuse(fixedSizeAuthID[:])
-			if burnErr != nil {
-				Autherr = newError("invalid auth, can't taint legacy userHash").Base(burnErr)
-			}
-			// It is possible that we are under attack described in https://github.com/xray/xray-core/issues/2523
-			return nil, drainConnection(Autherr)
-		} else {
-			return nil, newError("invalid auth, but this is a AEAD request")
-		}
+		return nil, errors.New("invalid auth, but this is a AEAD request")
 	}
 
 	if request.Address == nil {
-		return nil, newError("invalid remote address")
+		return nil, errors.New("invalid remote address")
 	}
 
 	if request.Security == protocol.SecurityType_UNKNOWN || request.Security == protocol.SecurityType_AUTO {
-		return nil, newError("unknown security type: ", request.Security)
+		return nil, errors.New("unknown security type: ", request.Security)
 	}
 
 	return request, nil
 }
 
 // DecodeRequestBody returns Reader from which caller can fetch decrypted body.
-func (s *ServerSession) DecodeRequestBody(request *protocol.RequestHeader, reader io.Reader) buf.Reader {
+func (s *ServerSession) DecodeRequestBody(request *protocol.RequestHeader, reader io.Reader) (buf.Reader, error) {
 	var sizeParser crypto.ChunkSizeDecoder = crypto.PlainChunkSizeParser{}
 	if request.Option.Has(protocol.RequestOptionChunkMasking) {
 		sizeParser = NewShakeSizeParser(s.requestBodyIV[:])
 	}
 	var padding crypto.PaddingLengthGenerator
 	if request.Option.Has(protocol.RequestOptionGlobalPadding) {
-		padding = sizeParser.(crypto.PaddingLengthGenerator)
+		var ok bool
+		padding, ok = sizeParser.(crypto.PaddingLengthGenerator)
+		if !ok {
+			return nil, errors.New("invalid option: RequestOptionGlobalPadding")
+		}
 	}
 
 	switch request.Security {
 	case protocol.SecurityType_NONE:
 		if request.Option.Has(protocol.RequestOptionChunkStream) {
 			if request.Command.TransferType() == protocol.TransferTypeStream {
-				return crypto.NewChunkStreamReader(sizeParser, reader)
+				return crypto.NewChunkStreamReader(sizeParser, reader), nil
 			}
 
 			auth := &crypto.AEADAuthenticator{
@@ -332,22 +273,9 @@ func (s *ServerSession) DecodeRequestBody(request *protocol.RequestHeader, reade
 				NonceGenerator:          crypto.GenerateEmptyBytes(),
 				AdditionalDataGenerator: crypto.GenerateEmptyBytes(),
 			}
-			return crypto.NewAuthenticationReader(auth, sizeParser, reader, protocol.TransferTypePacket, padding)
+			return crypto.NewAuthenticationReader(auth, sizeParser, reader, protocol.TransferTypePacket, padding), nil
 		}
-		return buf.NewReader(reader)
-
-	case protocol.SecurityType_LEGACY:
-		aesStream := crypto.NewAesDecryptionStream(s.requestBodyKey[:], s.requestBodyIV[:])
-		cryptionReader := crypto.NewCryptionReader(aesStream, reader)
-		if request.Option.Has(protocol.RequestOptionChunkStream) {
-			auth := &crypto.AEADAuthenticator{
-				AEAD:                    new(FnvAuthenticator),
-				NonceGenerator:          crypto.GenerateEmptyBytes(),
-				AdditionalDataGenerator: crypto.GenerateEmptyBytes(),
-			}
-			return crypto.NewAuthenticationReader(auth, sizeParser, cryptionReader, request.Command.TransferType(), padding)
-		}
-		return buf.NewReader(cryptionReader)
+		return buf.NewReader(reader), nil
 
 	case protocol.SecurityType_AES128_GCM:
 		aead := crypto.NewAesGcm(s.requestBodyKey[:])
@@ -367,7 +295,7 @@ func (s *ServerSession) DecodeRequestBody(request *protocol.RequestHeader, reade
 			}
 			sizeParser = NewAEADSizeParser(lengthAuth)
 		}
-		return crypto.NewAuthenticationReader(auth, sizeParser, reader, request.Command.TransferType(), padding)
+		return crypto.NewAuthenticationReader(auth, sizeParser, reader, request.Command.TransferType(), padding), nil
 
 	case protocol.SecurityType_CHACHA20_POLY1305:
 		aead, _ := chacha20poly1305.New(GenerateChacha20Poly1305Key(s.requestBodyKey[:]))
@@ -389,35 +317,27 @@ func (s *ServerSession) DecodeRequestBody(request *protocol.RequestHeader, reade
 			}
 			sizeParser = NewAEADSizeParser(lengthAuth)
 		}
-		return crypto.NewAuthenticationReader(auth, sizeParser, reader, request.Command.TransferType(), padding)
+		return crypto.NewAuthenticationReader(auth, sizeParser, reader, request.Command.TransferType(), padding), nil
 
 	default:
-		panic("Unknown security type.")
+		return nil, errors.New("invalid option: Security")
 	}
 }
 
 // EncodeResponseHeader writes encoded response header into the given writer.
 func (s *ServerSession) EncodeResponseHeader(header *protocol.ResponseHeader, writer io.Writer) {
 	var encryptionWriter io.Writer
-	if !s.isAEADRequest {
-		s.responseBodyKey = md5.Sum(s.requestBodyKey[:])
-		s.responseBodyIV = md5.Sum(s.requestBodyIV[:])
-	} else {
-		BodyKey := sha256.Sum256(s.requestBodyKey[:])
-		copy(s.responseBodyKey[:], BodyKey[:16])
-		BodyIV := sha256.Sum256(s.requestBodyIV[:])
-		copy(s.responseBodyIV[:], BodyIV[:16])
-	}
+	BodyKey := sha256.Sum256(s.requestBodyKey[:])
+	copy(s.responseBodyKey[:], BodyKey[:16])
+	BodyIV := sha256.Sum256(s.requestBodyIV[:])
+	copy(s.responseBodyIV[:], BodyIV[:16])
 
 	aesStream := crypto.NewAesEncryptionStream(s.responseBodyKey[:], s.responseBodyIV[:])
 	encryptionWriter = crypto.NewCryptionWriter(aesStream, writer)
 	s.responseWriter = encryptionWriter
 
 	aeadEncryptedHeaderBuffer := bytes.NewBuffer(nil)
-
-	if s.isAEADRequest {
-		encryptionWriter = aeadEncryptedHeaderBuffer
-	}
+	encryptionWriter = aeadEncryptedHeaderBuffer
 
 	common.Must2(encryptionWriter.Write([]byte{s.responseHeader, byte(header.Option)}))
 	err := MarshalCommand(header.Command, encryptionWriter)
@@ -425,49 +345,49 @@ func (s *ServerSession) EncodeResponseHeader(header *protocol.ResponseHeader, wr
 		common.Must2(encryptionWriter.Write([]byte{0x00, 0x00}))
 	}
 
-	if s.isAEADRequest {
-		aeadResponseHeaderLengthEncryptionKey := vmessaead.KDF16(s.responseBodyKey[:], vmessaead.KDFSaltConstAEADRespHeaderLenKey)
-		aeadResponseHeaderLengthEncryptionIV := vmessaead.KDF(s.responseBodyIV[:], vmessaead.KDFSaltConstAEADRespHeaderLenIV)[:12]
+	aeadResponseHeaderLengthEncryptionKey := vmessaead.KDF16(s.responseBodyKey[:], vmessaead.KDFSaltConstAEADRespHeaderLenKey)
+	aeadResponseHeaderLengthEncryptionIV := vmessaead.KDF(s.responseBodyIV[:], vmessaead.KDFSaltConstAEADRespHeaderLenIV)[:12]
 
-		aeadResponseHeaderLengthEncryptionKeyAESBlock := common.Must2(aes.NewCipher(aeadResponseHeaderLengthEncryptionKey)).(cipher.Block)
-		aeadResponseHeaderLengthEncryptionAEAD := common.Must2(cipher.NewGCM(aeadResponseHeaderLengthEncryptionKeyAESBlock)).(cipher.AEAD)
+	aeadResponseHeaderLengthEncryptionAEAD := crypto.NewAesGcm(aeadResponseHeaderLengthEncryptionKey)
 
-		aeadResponseHeaderLengthEncryptionBuffer := bytes.NewBuffer(nil)
+	aeadResponseHeaderLengthEncryptionBuffer := bytes.NewBuffer(nil)
 
-		decryptedResponseHeaderLengthBinaryDeserializeBuffer := uint16(aeadEncryptedHeaderBuffer.Len())
+	decryptedResponseHeaderLengthBinaryDeserializeBuffer := uint16(aeadEncryptedHeaderBuffer.Len())
 
-		common.Must(binary.Write(aeadResponseHeaderLengthEncryptionBuffer, binary.BigEndian, decryptedResponseHeaderLengthBinaryDeserializeBuffer))
+	common.Must(binary.Write(aeadResponseHeaderLengthEncryptionBuffer, binary.BigEndian, decryptedResponseHeaderLengthBinaryDeserializeBuffer))
 
-		AEADEncryptedLength := aeadResponseHeaderLengthEncryptionAEAD.Seal(nil, aeadResponseHeaderLengthEncryptionIV, aeadResponseHeaderLengthEncryptionBuffer.Bytes(), nil)
-		common.Must2(io.Copy(writer, bytes.NewReader(AEADEncryptedLength)))
+	AEADEncryptedLength := aeadResponseHeaderLengthEncryptionAEAD.Seal(nil, aeadResponseHeaderLengthEncryptionIV, aeadResponseHeaderLengthEncryptionBuffer.Bytes(), nil)
+	common.Must2(io.Copy(writer, bytes.NewReader(AEADEncryptedLength)))
 
-		aeadResponseHeaderPayloadEncryptionKey := vmessaead.KDF16(s.responseBodyKey[:], vmessaead.KDFSaltConstAEADRespHeaderPayloadKey)
-		aeadResponseHeaderPayloadEncryptionIV := vmessaead.KDF(s.responseBodyIV[:], vmessaead.KDFSaltConstAEADRespHeaderPayloadIV)[:12]
+	aeadResponseHeaderPayloadEncryptionKey := vmessaead.KDF16(s.responseBodyKey[:], vmessaead.KDFSaltConstAEADRespHeaderPayloadKey)
+	aeadResponseHeaderPayloadEncryptionIV := vmessaead.KDF(s.responseBodyIV[:], vmessaead.KDFSaltConstAEADRespHeaderPayloadIV)[:12]
 
-		aeadResponseHeaderPayloadEncryptionKeyAESBlock := common.Must2(aes.NewCipher(aeadResponseHeaderPayloadEncryptionKey)).(cipher.Block)
-		aeadResponseHeaderPayloadEncryptionAEAD := common.Must2(cipher.NewGCM(aeadResponseHeaderPayloadEncryptionKeyAESBlock)).(cipher.AEAD)
+	aeadResponseHeaderPayloadEncryptionAEAD := crypto.NewAesGcm(aeadResponseHeaderPayloadEncryptionKey)
 
-		aeadEncryptedHeaderPayload := aeadResponseHeaderPayloadEncryptionAEAD.Seal(nil, aeadResponseHeaderPayloadEncryptionIV, aeadEncryptedHeaderBuffer.Bytes(), nil)
-		common.Must2(io.Copy(writer, bytes.NewReader(aeadEncryptedHeaderPayload)))
-	}
+	aeadEncryptedHeaderPayload := aeadResponseHeaderPayloadEncryptionAEAD.Seal(nil, aeadResponseHeaderPayloadEncryptionIV, aeadEncryptedHeaderBuffer.Bytes(), nil)
+	common.Must2(io.Copy(writer, bytes.NewReader(aeadEncryptedHeaderPayload)))
 }
 
 // EncodeResponseBody returns a Writer that auto-encrypt content written by caller.
-func (s *ServerSession) EncodeResponseBody(request *protocol.RequestHeader, writer io.Writer) buf.Writer {
+func (s *ServerSession) EncodeResponseBody(request *protocol.RequestHeader, writer io.Writer) (buf.Writer, error) {
 	var sizeParser crypto.ChunkSizeEncoder = crypto.PlainChunkSizeParser{}
 	if request.Option.Has(protocol.RequestOptionChunkMasking) {
 		sizeParser = NewShakeSizeParser(s.responseBodyIV[:])
 	}
 	var padding crypto.PaddingLengthGenerator
 	if request.Option.Has(protocol.RequestOptionGlobalPadding) {
-		padding = sizeParser.(crypto.PaddingLengthGenerator)
+		var ok bool
+		padding, ok = sizeParser.(crypto.PaddingLengthGenerator)
+		if !ok {
+			return nil, errors.New("invalid option: RequestOptionGlobalPadding")
+		}
 	}
 
 	switch request.Security {
 	case protocol.SecurityType_NONE:
 		if request.Option.Has(protocol.RequestOptionChunkStream) {
 			if request.Command.TransferType() == protocol.TransferTypeStream {
-				return crypto.NewChunkStreamWriter(sizeParser, writer)
+				return crypto.NewChunkStreamWriter(sizeParser, writer), nil
 			}
 
 			auth := &crypto.AEADAuthenticator{
@@ -475,20 +395,9 @@ func (s *ServerSession) EncodeResponseBody(request *protocol.RequestHeader, writ
 				NonceGenerator:          crypto.GenerateEmptyBytes(),
 				AdditionalDataGenerator: crypto.GenerateEmptyBytes(),
 			}
-			return crypto.NewAuthenticationWriter(auth, sizeParser, writer, protocol.TransferTypePacket, padding)
+			return crypto.NewAuthenticationWriter(auth, sizeParser, writer, protocol.TransferTypePacket, padding), nil
 		}
-		return buf.NewWriter(writer)
-
-	case protocol.SecurityType_LEGACY:
-		if request.Option.Has(protocol.RequestOptionChunkStream) {
-			auth := &crypto.AEADAuthenticator{
-				AEAD:                    new(FnvAuthenticator),
-				NonceGenerator:          crypto.GenerateEmptyBytes(),
-				AdditionalDataGenerator: crypto.GenerateEmptyBytes(),
-			}
-			return crypto.NewAuthenticationWriter(auth, sizeParser, s.responseWriter, request.Command.TransferType(), padding)
-		}
-		return &buf.SequentialWriter{Writer: s.responseWriter}
+		return buf.NewWriter(writer), nil
 
 	case protocol.SecurityType_AES128_GCM:
 		aead := crypto.NewAesGcm(s.responseBodyKey[:])
@@ -508,7 +417,7 @@ func (s *ServerSession) EncodeResponseBody(request *protocol.RequestHeader, writ
 			}
 			sizeParser = NewAEADSizeParser(lengthAuth)
 		}
-		return crypto.NewAuthenticationWriter(auth, sizeParser, writer, request.Command.TransferType(), padding)
+		return crypto.NewAuthenticationWriter(auth, sizeParser, writer, request.Command.TransferType(), padding), nil
 
 	case protocol.SecurityType_CHACHA20_POLY1305:
 		aead, _ := chacha20poly1305.New(GenerateChacha20Poly1305Key(s.responseBodyKey[:]))
@@ -530,9 +439,9 @@ func (s *ServerSession) EncodeResponseBody(request *protocol.RequestHeader, writ
 			}
 			sizeParser = NewAEADSizeParser(lengthAuth)
 		}
-		return crypto.NewAuthenticationWriter(auth, sizeParser, writer, request.Command.TransferType(), padding)
+		return crypto.NewAuthenticationWriter(auth, sizeParser, writer, request.Command.TransferType(), padding), nil
 
 	default:
-		panic("Unknown security type.")
+		return nil, errors.New("invalid option: Security")
 	}
 }
